<?php

namespace Drupal\monarch_hook_plugin\Plugin\Hook;

/**
 * A plugin to allow preprocess to use the hook plugin system.
 *
 * @Hook(
 *   id = "monarch_hook_plugin_preprocess_engine",
 * )
 */
class Preprocess extends HookBase {

  /**
   * Implements hook_theme_registry_alter().
   */
  public function hookThemeRegistryAlter(&$theme_registry) {
    foreach ($theme_registry as &$entry) {
      if (is_array($entry['preprocess functions'] ?? NULL)) {
        if (empty($entry['preprocess functions'])) {
          $entry['preprocess functions'] = [
            '\\' . $this::class . '::prePreprocess',
            '\\' . $this::class . '::postPreprocess',
          ];
        }
        else {
          $first = array_shift($entry['preprocess functions']);
          $entry['preprocess functions'] = [
            $first,
            '\\' . $this::class . '::prePreprocess',
            ...$entry['preprocess functions'],
            '\\' . $this::class . '::postPreprocess',
          ];
        }
      }
    }
  }

  /**
   * Pre-preprocess templates.
   */
  public static function prePreprocess(&$variables, $hook, $info) {
    $manager = \Drupal::service('plugin.manager.monarch_hook_plugin.hook');
    foreach ([
      'hook_preprocess',
      'template_preprocess',
      'preprocess',
      'hook_preprocess_' . $hook,
      'template_preprocess_' . $hook,
      'preprocess_' . $hook,
    ] as $hook_name) {
      $func_name = $manager->getFunctionName($hook_name);

      foreach ($manager->getPlugins($func_name, -INF, 0) as $plugin) {
        $plugin->{$func_name}($variables, $hook, $info);
      }
    }
  }

  /**
   * Post-preprocess templates.
   */
  public static function postPreprocess(&$variables, $hook, $info) {
    $manager = \Drupal::service('plugin.manager.monarch_hook_plugin.hook');
    foreach ([
      'hook_preprocess',
      'template_preprocess',
      'preprocess',
      'hook_preprocess_' . $hook,
      'template_preprocess_' . $hook,
      'preprocess_' . $hook,
    ] as $hook_name) {
      $func_name = $manager->getFunctionName($hook_name);

      foreach ($manager->getPlugins($func_name, 0, INF) as $plugin) {
        $plugin->{$func_name}($variables, $hook, $info);
      }
    }
  }

}
